package me.schlaubi.fluttercontactpicker

import android.app.Activity
import android.content.Intent
import android.database.Cursor
import android.net.Uri
import android.provider.ContactsContract
import androidx.annotation.NonNull
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry

class FlutterContactPickerPlugin(private val activity: Activity) : MethodCallHandler, PluginRegistry.ActivityResultListener {

    private var pendingResult: Result? = null

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
            "pickPhoneContact" -> requestPicker(PICK_PHONE, ContactsContract.CommonDataKinds.Phone.CONTENT_URI, result)
            "pickEmailContact" -> requestPicker(PICK_EMAIL, ContactsContract.CommonDataKinds.Email.CONTENT_URI, result)
            else -> result.notImplemented()
        }
    }

    private fun requestPicker(requestCode: Int, type: Uri, result: Result) {
        assert(pendingResult == null) { "Multiple requests are not supported" }
        val pickerIntent = Intent(Intent.ACTION_PICK, type)
        activity.startActivityForResult(pickerIntent, requestCode)
        pendingResult = result
    }

    private fun buildPhoneNumber(cursor: Cursor): Map<String, String> {
        val phoneType = cursor.getInt(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.TYPE))
        val customLabel = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.LABEL))
        val label = ContactsContract.CommonDataKinds.Phone.getTypeLabel(activity.resources, phoneType, customLabel) as String
        val number = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
        return mapOf("phoneNumber" to number, "label" to label)
    }

    private fun buildEmailAddress(cursor: Cursor): Map<String, String> {
        val phoneType = cursor.getInt(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.TYPE))
        val customLabel = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.LABEL))
        val label = ContactsContract.CommonDataKinds.Email.getTypeLabel(activity.resources, phoneType, customLabel) as String
        val address = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Email.DATA))
        return mapOf("email" to address, "label" to label)
    }

    private fun processContact(intent: Intent, dataName: String, dataProcessor: (Cursor) -> Map<String, String>) {
        val data = intent.data!!
        activity.contentResolver.query(data, null, null, null, null).use {
            require(it != null) { "Cursor must not be null" }
            it.moveToFirst()
            val processedData = dataProcessor(it)
            val fullName = it.getString(it.getColumnIndex(ContactsContract.CommonDataKinds.Identity.DISPLAY_NAME))
            val result = mapOf("fullName" to fullName, dataName to processedData)
            pendingResult!!.success(result)
            pendingResult = null
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent): Boolean {
        when (requestCode) {
            PICK_EMAIL -> processContact(data, "email", ::buildEmailAddress)
            PICK_PHONE -> processContact(data, "phoneNumber", ::buildPhoneNumber)
            else -> return false
        }
        return true
    }

    companion object {
        @JvmStatic
        fun registerWith(registrar: PluginRegistry.Registrar) {
            val channel = MethodChannel(registrar.messenger(), "fluttercontactpicker")
            val plugin = FlutterContactPickerPlugin(registrar.activity())
            channel.setMethodCallHandler(plugin)
            registrar.addActivityResultListener(plugin)
        }

        private const val PICK_PHONE = 2015
        private const val PICK_EMAIL = 2020
    }
}
